package edu.sjsu.rhinofaces;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.net.URL;
import java.util.logging.Logger;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.Scriptable;

/**
 * 
 * @author Tom Austin
 */
public class JavaScriptEnvironment
{
   private static Logger logger = Logger.global;

   private Scriptable scope;
   
   private JSVar lastVariableCalled;

   /**
      Constructor.  Initializes the JS environment.
    */
   public JavaScriptEnvironment()
   {
      super();
      Context jsContext = Context.enter();
      try
      {
         scope = jsContext.initStandardObjects();
      }
      finally
      {
         Context.exit();
      }
      readControllers(scope);
      
      this.lastVariableCalled = null;
   }

   /**
      Reads all of the controllers from the controllers/ directory on
      the classpath.
      
      @param scope
    */
   private static void readControllers(Scriptable scope)
   {
      try
      {
         URL controllersDirectory =
               JavaScriptEnvironment.class.getClassLoader().getResource("controllers/");
         File dir = new File(controllersDirectory.getPath());
         for (File script : dir.listFiles())
         {
            String lowerFileName = script.getName().toLowerCase();
            // Either .js or .rhino will be considered OK names for controllers
            if (lowerFileName.endsWith(".js") || lowerFileName.endsWith(".rhino"))
            {
               executeScript(scope, script);
            }
         }
      }
      catch (IOException e)
      {
         // TODO Auto-generated catch block
         e.printStackTrace();
      }
   }
   
   private static void executeScript(Scriptable scope, File script) throws IOException
   {
      try
      {
         Context ctxt = Context.enter();
         logger.info("Loading controller " + script.getAbsolutePath());
         Reader scriptReader = new FileReader(script);
         // Can't figure out how to get the line number when doing this,
         // so setting it to 0.
         ctxt.evaluateReader(scope, scriptReader, script.getAbsolutePath(), 0, null);
      }
      finally
      {
         Context.exit();
      }
   }
   
   /**
    * Executes arbitrary Rhino commands and returns the result. This is
    * restricted to other items withing the package.
    */
   public Object evalRhino(String expression)
   {
      logger.info("JS: " + expression);
      Context jsContext = Context.enter();
      Object result = null;
      try
      {
         result = jsContext.evaluateString(scope, expression, "<jsfRuntime>",
               1, null);
      }
      finally
      {
         Context.exit();
      }

      if (result == org.mozilla.javascript.Undefined.instance || result == null)
      {
         return "";
      }
      else
      {
         return result.toString();
      }
   }
   
   /**
      @param property
    */
   public void initializeVariable(Object property)
   {
      String var = "" + property;
      if (!scope.has(var, scope))
      {
         evalRhino("var " + var + " = new " + capitalize(var) + "();");
      }
   }
   
   /**
      Capitalizes the first letter of the specified string.
      @param str
      @return
    */
   private String capitalize(String str)
   {
      return str.toUpperCase().charAt(0) + str.substring(1);
   }

   /**
      @param property
      @return
    */
   public boolean hasVar(Object property)
   {
      if (!(property instanceof String)) return false;
      return "quiz".equals(property);//scope.has((String)property, scope);
   }
   
   //Track the last variable called.  This is a hack to catch the JS calls.
   public JSVar getLastVariableCalled() { return this.lastVariableCalled; }
   public void setLastVariableCalled(JSVar lastVariableCalled) { this.lastVariableCalled = lastVariableCalled; }
}
